#!/usr/bin/sudo bash

#$1 - file-name

set -e

if [ $# -ne 1 ]; then
  echo "Usage: $0 <file-name>"
  exit 1
fi

renice 19 -p $$ &> /dev/null

FILE_NAME=$1.tar.gz
DIR=saved_files

#clean up before we start
cd /tmp;
/bin/rm -rf $DIR *.tar.gz 2> /dev/null
cd - > /dev/null

mkdir -p /tmp/$DIR
mkdir -p /tmp/$DIR/cores

function handle_ctrl_c {
  echo -e "\nCaught Ctrl+C, cleaning up and exiting..."
  cleanup_and_exit 1
}

trap handle_ctrl_c SIGINT

function cleanup_and_exit {
  exit_code=$1
  if [ -z $1 ]
  then
    exit_code=0
  fi

  /bin/rm -rf $DIR 2> /dev/null
  exit $1
}

function create_tar {
  available_space_mb=`df / -m | grep /$ | awk '{print $4}'`
  files_disk_space=`du -sm $DIR | awk '{print $1}'`
  required_space=$((files_disk_space * 2))
  minimal_disk_space=100 # 100MB

  if [ $required_space -gt $((available_space_mb - minimal_disk_space)) ]
  then
    echo "Not enough disk space to compress TMA logs"
    cleanup_and_exit 1
  fi

  /bin/tar czf $FILE_NAME $DIR/ 2> /dev/null
}

function copy_file {
  # Iterate all args and extract all args except the last as source, and the last as destination
  unset src_files
  while [ $# -gt 1 ]
  do
    src_files+=("$1")
    shift
  done

  src=${src_files[*]}
  dst=$1

  available_space_mb=`df -m / | grep /$ | awk '{print $4}'`
  required_space=`du -sm $src 2>/dev/null | awk '{sum+=$1} END {printf "%d", sum}'`
  minimal_disk_space=100 # 100MB

  if [[ $required_space -gt $((available_space_mb - minimal_disk_space)) ]]
  then
    echo "Not enough disk space to collect TMA logs"
    cleanup_and_exit 1
  fi

  /bin/cp -r $src $dst 2>/dev/null || :;
}

function copy_files {
  TMA_DIR=$1
  TOSA_DIR=tosa
  mkdir -p /tmp/$DIR/$TMA_DIR;

  #copy conf files
  copy_file /opt/$TMA_DIR/tma.conf /tmp/$DIR/$TMA_DIR
  copy_file /opt/$TMA_DIR/tma_conf.json /tmp/$DIR/$TMA_DIR

  #copy all logs
  copy_file /var/log/$TMA_DIR/*.log* /tmp/$DIR/$TMA_DIR
  copy_file /var/log/$TOSA_DIR/*.log* /tmp/$DIR/$TMA_DIR

  #copy system logs
  copy_file /var/log/{dmesg*,messages*,secure*,yum.log*,pluto*} /tmp/$DIR/$TMA_DIR

  #copy SELinux alerts logs
  copy_file /var/log/audit/audit.log* /tmp/$DIR/$TMA_DIR
}

#loop through all TMA folders in /opt/
echo "$TMA_DIR";
for f in /opt/*;
do
  if [[ ${f} == /opt/tma* ]] && [[ -d "$f" && ! -L "$f" ]];
  then
      copy_files $(basename $f);
  fi
done

#copy core dump files
copy_file  /var/log/core.* /tmp/$DIR/cores

#create tar.gz file
cd /tmp;
create_tar

#clean up
cleanup_and_exit
